/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.ScriptExecution.java
 * Created on: Jun 12, 2016
 */
package fr.bigeon.gclc.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.List;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.GCLCConstants;
import fr.bigeon.gclc.exception.CommandParsingException;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;

/** A command that will laucnh a series of command from a file
 * <p>
 * This command will read a file and execute each non empty non commented line
 * as a command of the application.
 *
 * @author Emmanuel Bigeon */
public class ScriptExecution extends Command {

    /** The application */
    private final ConsoleApplication application;
    /** The commenting prefix */
    private final String commentPrefix;

    /** @param name the name of the command
     * @param application the application
     * @param commentPrefix the comment prefix in the script files */
    public ScriptExecution(String name, ConsoleApplication application,
            String commentPrefix) {
        super(name);
        this.application = application;
        this.commentPrefix = commentPrefix;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#execute(java.lang.String[]) */
    @SuppressWarnings("resource")
    @Override
    public void execute(String... args) throws CommandRunException {
        if (args.length == 0) {
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    "Expecting a file", this); //$NON-NLS-1$
        }
        List<String> argsList = Arrays.asList(args);
        String scriptFile = argsList.remove(0);
        Object[] params = argsList.toArray();
        try (FileReader fReader = new FileReader(new File(scriptFile));
             BufferedReader reader = new BufferedReader(fReader)) {
            String cmd;
            for (int i = 1; i < args.length; i++) {
                params[i - 1] = args[i];
            }
            while ((cmd = reader.readLine()) != null) {
                if (cmd.startsWith(" ") || cmd.startsWith("\t")) { //$NON-NLS-1$ //$NON-NLS-2$
                    reader.close();
                    fReader.close();
                    throw new CommandRunException(
                            "Invalid command in script (line starts with space character)", //$NON-NLS-1$
                            this);
                }
                if (cmd.isEmpty() || cmd.startsWith(commentPrefix)) {
                    continue;
                }
                String cmdLine = MessageFormat.format(cmd, params);
                List<String> ps = GCLCConstants.splitCommand(cmdLine);
                String command = ps.remove(0);
                application.executeSub(command, ps.toArray(new String[0]));
            }
        } catch (CommandParsingException e) {
            throw new CommandRunException("Invalid command in script (" + //$NON-NLS-1$
                                          e.getLocalizedMessage() + ")", //$NON-NLS-1$
                    e, this);
        } catch (IOException e) {
            throw new CommandRunException(
                    "Unable to read script (" + e.getLocalizedMessage() + ")", //$NON-NLS-1$ //$NON-NLS-2$
                    e, this);
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        StringBuilder builder = new StringBuilder();
        builder.append(
                "  scriptfile: path to the file containing the script to execute."); //$NON-NLS-1$
        builder.append(System.lineSeparator());
        builder.append(System.lineSeparator());
        builder.append(
                "  The script file must contain one line commands. The lines must never"); //$NON-NLS-1$
        builder.append(System.lineSeparator());
        builder.append(
                "start with whitespace characters. The lines starting with"); //$NON-NLS-1$
        builder.append(System.lineSeparator());
        builder.append("\"" + commentPrefix + //$NON-NLS-1$
                       "\" will be ignored as well as empty lines."); //$NON-NLS-1$
        builder.append(System.lineSeparator());

        return builder.toString();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Execute a script"; //$NON-NLS-1$
    }

}
